
#include "comb/necklace-mod-color.h"

//#include "comb/is-setpart-rgs.h"  // Lyndon word pass is_setpart_rgs

#include "jjassert.h"

#include "nextarg.h"
#include "fxttypes.h"  // ulong

#include "fxtio.h"

//% Generate all pre-necklaces, necklaces, and Lyndon words with a given number of colors.
//% The underlying equivalence relation additionally contains all permutations of colors (numbers).

// OEIS sequences:
//
// Lyndon words:
// echo $(for n in $(seq 1 12); do ./bin $n ; done | g yndon | cut -d= -f4 | cut -d\# -f1) | sed 's/ /, /g;'
// A254036 (Number of Lyndon-Bell strings of length n)
//
// necklaces:
// echo $(for n in $(seq 1 12); do ./bin $n ; done | g yndon | cut -d= -f3 | cut -d\# -f1) | sed 's/ /, /g;'
// 1, 2, 4, 11, 34, 133, 565, 2692, 13849, 76696, 453026, 2840111 = A000000
//
// m = 3:
// echo $(for n in $(seq 1 12); do ./bin $n 3; done | g yndon | cut -d= -f3 | cut -d\# -f1) | sed 's/ /, /g;'
// necklaces: 1, 2, 4, 10, 24, 66, 172, 474, 1289, 3560, 9820, 27327 = A000000
// Lyndon words: 1, 1, 3, 8, 23, 61, 171, 464, 1285, 3535, 9819, 27253 = A000000

// Cf. comb/necklace-demo.cc


#define PRX()  { if (x[k])  cout << x[k]; else cout << '.'; }

//#define TIMING  // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 5;
    NXARG(n, "length of strings >=1");
    ulong m = n;
    NXARG(m, "number of different beads >=2");

    necklace_mod_color P(n, m);

    const ulong *x = P.data();
    (void)x;

    ulong j = P.j_;  // ==1
    ulong pct = 0;  // count pre-necklaces (pre-prime strings)
    ulong ct = 0;   // count necklaces (prime strings)
    ulong lct = 0;  // count Lyndon words
    do
    {
        ++pct;
        ct += P.is_necklace();
        lct += P.is_lyndon_word();

#if defined TIMING
#else  // TIMING

//        if ( ! P.is_necklace() )  continue;  // don't show pre-necklaces
//        if ( ! P.is_lyn() )  continue;  // only show Lyndon words

        cout << setw(4) << ct << ":  ";
        // print full string:
        for (ulong k=0; k<n; ++k)  PRX();
        cout << "  " << setw(2) << j;  // period

//        cout << "    ";  P.debug_print();

        if ( P.is_necklace() ) cout << "  N";  // is a necklace
        if ( P.is_lyndon_word() ) cout << " L";  // is a Lyndon word
        cout << endl;

        jjassert( P.OK() );
#endif  // TIMING
    }
    while ( (j=P.next_pre()) );

    cout << "  # pre-necklaces = " << pct;
    cout << ",  # necklaces = " << ct;
    cout << ",  # Lyndon words = " << lct;
    cout << endl;

    return 0;
}
// -------------------------

/*
Timing:

Timing: (Intel(R) Core(TM) i7-8700K CPU @ 3.70GHz)

GCC 12.2

time ./bin 15
arg 1: 15 == n  [length of strings >=1]  default=5
arg 2: 15 == m  [number of different beads >=2]  default=15
  # pre-necklaces = 1110480416  # necklaces = 957267578  # Lyndon words = 957267541
3.18user 0.00system 0:03.18elapsed 99%CPU
 ==> 957267578/3.18 == 301,027,540 necklaces per second
*/

/*
  OEIS sequences:
  m = 3:

  necklaces:
  echo $(for n in $(seq 1 12); do ./bin $n 3; done | g yndon | cut -d= -f3 | cut -d\# -f1) | sed 's/ /, /g;'
  1, 2, 4, 10, 24, 66, 172, 474, 1289, 3560, 9820, 27327 = A000000

  Lyndon words:
  echo $(for n in $(seq 1 12); do ./bin $n 3; done | g yndon | cut -d= -f4 | cut -d\# -f1) | sed 's/ /, /g;'
  1, 1, 3, 8, 23, 61, 171, 464, 1285, 3535, 9819, 27253 = A000000

  m = infty:

  1, 1, 3, 9, 33, 128, 564, 2681, 13845, 76661, 453025, 2839969 = A254036
  "Number of Lyndon-Bell strings of length n."

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/necklace-mod-color-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/necklace-mod-color-demo.cc DEMOFLAGS=-DTIMING"
/// End:

